# 1.3. Deploy TeraFlowSDN over MicroK8s

This section describes how to deploy TeraFlowSDN controller on top of MicroK8s using the 
environment configured in the previous sections.


## 1.3.1. Install prerequisites
```bash
sudo apt-get install -y git curl jq
```


## 1.3.2. Clone the Git repository of the TeraFlowSDN controller
Clone from ETSI-hosted GitLab code repository:
```bash
mkdir ~/tfs-ctrl
git clone https://labs.etsi.org/rep/tfs/controller.git ~/tfs-ctrl
```

__Important__: The original H2020-TeraFlow project hosted on GitLab.com has been 
archieved and will not receive further contributions/updates.
Please, clone from [ETSI-hosted GitLab code repository](https://labs.etsi.org/rep/tfs/controller).


## 1.3.3. Checkout the appropriate Git branch
By default the *master* branch is checked out.
If you want to deploy the *develop* branch, that incorporates the most up-to-date code
contributions and features, run the following command:
```bash
cd ~/tfs-ctrl
git checkout develop
```


## 1.3.4. Prepare a deployment script with the deployment settings
Create a new deployment script, e.g., `my_deploy.sh`, adding the appropriate settings as 
follows.
This script, by default, makes use of the private Docker registry enabled in MicroK8s, 
as specified in `TFS_REGISTRY_IMAGE`.
It builds the Docker images for the subset of components defined in `TFS_COMPONENTS`, 
tags them with the tag defined in `TFS_IMAGE_TAG`, deploys them in the namespace defined 
in `TFS_K8S_NAMESPACE`, and (optionally) deploys the extra Kubernetes manifests listed 
in `TFS_EXTRA_MANIFESTS`. 
Besides, it lets you specify in `TFS_GRAFANA_PASSWORD` the password to be set for the 
Grafana `admin` user.

```bash
cd ~/tfs-ctrl
tee my_deploy.sh >/dev/null << EOF
export TFS_REGISTRY_IMAGE="http://localhost:32000/tfs/"
export TFS_COMPONENTS="context device automation pathcomp service slice compute monitoring webui"
export TFS_IMAGE_TAG="dev"
export TFS_K8S_NAMESPACE="tfs"
export TFS_EXTRA_MANIFESTS="manifests/nginx_ingress_http.yaml"
export TFS_GRAFANA_PASSWORD="admin123+"
EOF
```


## 1.3.5. Deploy TFS controller
First, source the deployment settings defined in the previous section.
This way, you do not need to specify the environment variables in each and every command 
you execute to operate the TFS controller.
Be aware to re-source the file if you open new terminal sessions.
Then, run the following command to deploy TeraFlowSDN controller on top of the MicroK8s 
Kubernetes platform.

```bash
cd ~/tfs-ctrl
source my_deploy.sh
./deploy.sh
```

The script performs the following steps:
1. Builds the Docker images for the components defined in `TFS_COMPONENTS`
2. Tags the Docker images with the value of `TFS_IMAGE_TAG`
3. Pushes the Docker images to the repository defined in `TFS_REGISTRY_IMAGE`
4. Creates the namespace defined in `TFS_K8S_NAMESPACE`
5. Deploys the components defined in `TFS_COMPONENTS`
6. Creates the file `tfs_runtime_env_vars.sh` with the environment variables for the components defined in `TFS_COMPONENTS` defining their local host addresses and their port numbers.
7. Create an ingress controller listening at port 80 for HTTP connections to enable external access to the TeraFlowSDN WebUI, Grafana Dashboards, Context Debug endpoints, and Compute NBI interfaces.
8. Initialize and configure the Grafana dashboards
9. Report a summary of the deployment (see 
   [1.5. Show Deployment and Log per Component](./1-5-deploy-logs-troubleshooting.md))
